/*
 * Extracted 2009 from mplayer 2009-02-10 libavcodec/bitstream.h.
 *
 * copyright (c) 2004 Michael Niedermayer <michaelni@gmx.at>
 *
 * Licensed under the GNU Lesser General Public License.
 * For licencing details see COPYING.LIB.
 */

/** \file bitstream.h Bitstream structures and inline functions. */

/** Structure for bistream I/O. */
struct getbit_context {
	/** Start of the internal buffer. */
	const uint8_t *buffer;
	/** End of the internal buffer. */
	const uint8_t *buffer_end;
	/** Bit counter. */
	int index;
};

#define VLC_TYPE int16_t

/** A variable length code table. */
struct vlc {
	/** Number of bits of the table. */
	int bits;
	/** The code and the bits table. */
	VLC_TYPE(*table)[2];
	/** The size of the table. */
	int table_size;
	/** Amount of memory allocated so far. */
	int table_allocated;
};

static inline uint32_t show_bits(struct getbit_context *gbc, int num)
{
	int idx = gbc->index;
	const uint8_t *p = gbc->buffer + (idx >> 3);
	uint32_t x = ((p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3]);
	return (x << (idx & 7)) >> (32 - num);
}

static inline int get_bits_count(struct getbit_context *gbc)
{
	return gbc->index;
}

static inline void skip_bits(struct getbit_context *gbc, int n)
{
	gbc->index += n;
}

static inline unsigned int get_bits(struct getbit_context *gbc, int n)
{
	unsigned int ret = show_bits(gbc, n);
	skip_bits(gbc, n);
	return ret;
}

/* This is rather hot, we can do better than get_bits(gbc, 1). */
static inline unsigned int get_bit(struct getbit_context *gbc)
{
	int idx = gbc->index++;
	uint8_t tmp = gbc->buffer[idx >> 3], mask = (1 << (7 - (idx & 7)));
	return !!(tmp & mask);
}

/**
 * Initialize a getbit_context structure.
 *
 * \param gbc The structure to initialize.
 * \param buffer The bitstream buffer.
 * \param size The size of the buffer in bytes.
 *
 * The bitstream buffer must be 4 bytes larger then the actual read bits
 * because the bitstream reader might read 32 bits at once and could read over
 * the end.
 */
static inline void init_get_bits(struct getbit_context *gbc,
		const uint8_t *buffer, int size)
{
	gbc->buffer = buffer;
	gbc->buffer_end = buffer + size;
	gbc->index = 0;
}

void init_vlc(struct vlc *vlc, int nb_bits, int nb_codes, const void *bits,
		const void *codes, int codes_size);
void free_vlc(struct vlc *vlc);
int get_vlc(struct getbit_context *gbc, VLC_TYPE(*table)[2], int bits,
		int max_depth);
